Trait CloneableByteSlice

unsafe trait CloneableByteSlice: ByteSlice + Clone

A ByteSlice which can be cloned without violating dereference stability.

Safety

If B: CloneableByteSlice, then the dereference stability properties required by ByteSlice (see that trait's safety documentation) do not only hold regarding two calls to b.deref() or b.deref_mut(), but also hold regarding c.deref() or c.deref_mut(), where c is produced by b.clone(), b.clone().clone(), etc.

Implementors