Struct F64

struct F64<O>(_, _)

A 64-bit floating point number stored in a given byte order.

F64 is like the native f64 type with two major differences: First, it has no alignment requirement (its alignment is 1). Second, the endianness of its memory layout is given by the type parameter O, which can be any type which implements ByteOrder. In particular, this refers to BigEndian, LittleEndian, NativeEndian, and NetworkEndian.

An F64 can be constructed using the new method, and its contained value can be obtained as a native f64 using the get method, or updated in place with the set method. In all cases, if the endianness O is not the same as the endianness of the current platform, an endianness swap will be performed in order to uphold the invariants that a) the layout of F64 has endianness O and that, b) the layout of f64 has the platform's native endianness.

F64 implements FromBytes, IntoBytes, and Unaligned, making it useful for parsing and serialization. See the module documentation for an example of how it can be used for parsing UDP packets.

Implementations

impl<O> F64<O>

const fn from_bytes(bytes: [u8; 8]) -> F64<O>

Constructs a new value from bytes which are already in O byte order.

const fn to_bytes(self: Self) -> [u8; 8]

Extracts the bytes of self without swapping the byte order.

The returned bytes will be in O byte order.

impl<O: ByteOrder> F64<O>

const fn new(n: f64) -> F64<O>

Constructs a new value, possibly performing an endianness swap to guarantee that the returned value has endianness O.

const fn get(self: Self) -> f64

Returns the value as a primitive type, possibly performing an endianness swap to guarantee that the return value has the endianness of the native platform.

fn set(self: &mut Self, n: f64)

Updates the value in place as a primitive type, possibly performing an endianness swap to guarantee that the stored value has the endianness O.

impl<O> AsMut for F64<O>

fn as_mut(self: &mut Self) -> &mut [u8; 8]

impl<O> AsRef for F64<O>

fn as_ref(self: &Self) -> &[u8; 8]

impl<O> Default for F64<O>

fn default() -> F64<O>

impl<O> Freeze for F64<O>

impl<O> FromBytes for F64<O>

impl<O> FromZeros for F64<O>

impl<O> Immutable for F64<O>

impl<O> IntoBytes for F64<O>

impl<O> KnownLayout for F64<O>

impl<O> PartialEq for F64<O>

fn eq(self: &Self, other: &[u8; 8]) -> bool

impl<O> RefUnwindSafe for F64<O>

impl<O> Send for F64<O>

impl<O> StructuralPartialEq for F64<O>

impl<O> Sync for F64<O>

impl<O> TryFromBytes for F64<O>

impl<O> Unaligned for F64<O>

impl<O> Unpin for F64<O>

impl<O> UnsafeUnpin for F64<O>

impl<O> UnwindSafe for F64<O>

impl<O: $crate::clone::Clone> Clone for F64<O>

fn clone(self: &Self) -> F64<O>

impl<O: $crate::cmp::Eq> Eq for F64<O>

impl<O: $crate::cmp::PartialEq> PartialEq for F64<O>

fn eq(self: &Self, other: &F64<O>) -> bool

impl<O: $crate::hash::Hash> Hash for F64<O>

fn hash<__H: $crate::hash::Hasher>(self: &Self, state: &mut __H)

impl<O: $crate::marker::Copy> Copy for F64<O>

impl<O: ByteOrder> Add for F64<O>

fn add(self: Self, rhs: f64) -> F64<O>

impl<O: ByteOrder> Add for F64<O>

fn add(self: Self, rhs: F64<O>) -> F64<O>

impl<O: ByteOrder> AddAssign for F64<O>

fn add_assign(self: &mut Self, rhs: f64)

impl<O: ByteOrder> AddAssign for F64<O>

fn add_assign(self: &mut Self, rhs: F64<O>)

impl<O: ByteOrder> Debug for F64<O>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<O: ByteOrder> Display for F64<O>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<O: ByteOrder> Div for F64<O>

fn div(self: Self, rhs: f64) -> F64<O>

impl<O: ByteOrder> Div for F64<O>

fn div(self: Self, rhs: F64<O>) -> F64<O>

impl<O: ByteOrder> DivAssign for F64<O>

fn div_assign(self: &mut Self, rhs: f64)

impl<O: ByteOrder> DivAssign for F64<O>

fn div_assign(self: &mut Self, rhs: F64<O>)

impl<O: ByteOrder> From for F64<O>

fn from(bytes: [u8; 8]) -> F64<O>

impl<O: ByteOrder> From for F64<O>

fn from(x: f64) -> F64<O>

impl<O: ByteOrder> Mul for F64<O>

fn mul(self: Self, rhs: f64) -> F64<O>

impl<O: ByteOrder> Mul for F64<O>

fn mul(self: Self, rhs: F64<O>) -> F64<O>

impl<O: ByteOrder> MulAssign for F64<O>

fn mul_assign(self: &mut Self, rhs: f64)

impl<O: ByteOrder> MulAssign for F64<O>

fn mul_assign(self: &mut Self, rhs: F64<O>)

impl<O: ByteOrder> Neg for F64<O>

fn neg(self: Self) -> F64<O>

impl<O: ByteOrder> PartialEq for F64<O>

fn eq(self: &Self, other: &f64) -> bool

impl<O: ByteOrder> PartialOrd for F64<O>

fn partial_cmp(self: &Self, other: &Self) -> Option<Ordering>

impl<O: ByteOrder> Rem for F64<O>

fn rem(self: Self, rhs: F64<O>) -> F64<O>

impl<O: ByteOrder> Rem for F64<O>

fn rem(self: Self, rhs: f64) -> F64<O>

impl<O: ByteOrder> RemAssign for F64<O>

fn rem_assign(self: &mut Self, rhs: f64)

impl<O: ByteOrder> RemAssign for F64<O>

fn rem_assign(self: &mut Self, rhs: F64<O>)

impl<O: ByteOrder> Sub for F64<O>

fn sub(self: Self, rhs: F64<O>) -> F64<O>

impl<O: ByteOrder> Sub for F64<O>

fn sub(self: Self, rhs: f64) -> F64<O>

impl<O: ByteOrder> SubAssign for F64<O>

fn sub_assign(self: &mut Self, rhs: f64)

impl<O: ByteOrder> SubAssign for F64<O>

fn sub_assign(self: &mut Self, rhs: F64<O>)

impl<O: ByteOrder, P: ByteOrder> From for F64<P>

fn from(x: F32<O>) -> F64<P>

impl<T> Any for F64<O>

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for F64<O>

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for F64<O>

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for F64<O>

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for F64<O>

fn from(t: T) -> T

Returns the argument unchanged.

impl<T, U> Into for F64<O>

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for F64<O>

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for F64<O>

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>