Trait ByteSliceMut

trait ByteSliceMut: ByteSlice + DerefMut

A mutable reference to a byte slice.

ByteSliceMut abstracts over various ways of storing a mutable reference to a byte slice, and is implemented for various special reference types such as RefMut<[u8]>.

ByteSliceMut is a shorthand for ByteSlice and DerefMut.

Implementors