Trait IntoByteSliceMut
unsafe trait IntoByteSliceMut<'a>: IntoByteSlice<'a> + ByteSliceMut
A ByteSliceMut that conveys no ownership, and so can be converted into a
mutable byte slice.
Some ByteSliceMut types (notably, the standard library's RefMut type)
convey ownership, and so they cannot soundly be moved by-value into a byte
slice type (&mut [u8]). Some methods in this crate's API (such as
Ref::into_mut) are only compatible with ByteSliceMut types without
these ownership semantics.
Required Methods
fn into_byte_slice_mut(self: Self) -> &'a mut [u8]Coverts
selfinto a&mut [u8].Safety
The returned reference has the same address and length as
self.deref()andself.deref_mut().Note that, combined with the safety invariant on
ByteSlice, this safety invariant implies that the returned reference is "stable" in the sense described in theByteSlicedocs.
Implementors
impl<'a> IntoByteSliceMut for &'a mut [u8]