Struct F32
struct F32<O>(_, _)
A 32-bit floating point number stored in a given byte order.
F32 is like the native f32 type with
two major differences: First, it has no alignment requirement (its alignment is 1).
Second, the endianness of its memory layout is given by the type parameter O,
which can be any type which implements ByteOrder. In particular, this refers
to BigEndian, LittleEndian, NativeEndian, and NetworkEndian.
An F32 can be constructed using
the new method, and its contained value can be obtained as a native
f32 using the get method, or updated in place with
the set method. In all cases, if the endianness O is not the same as the
endianness of the current platform, an endianness swap will be performed in
order to uphold the invariants that a) the layout of F32
has endianness O and that, b) the layout of f32 has
the platform's native endianness.
F32 implements FromBytes, IntoBytes, and Unaligned,
making it useful for parsing and serialization. See the module documentation for an
example of how it can be used for parsing UDP packets.
Implementations
impl<O> F32<O>
const fn from_bytes(bytes: [u8; 4]) -> F32<O>Constructs a new value from bytes which are already in
Obyte order.const fn to_bytes(self: Self) -> [u8; 4]Extracts the bytes of
selfwithout swapping the byte order.The returned bytes will be in
Obyte order.
impl<O: ByteOrder> F32<O>
const fn new(n: f32) -> F32<O>Constructs a new value, possibly performing an endianness swap to guarantee that the returned value has endianness
O.const fn get(self: Self) -> f32Returns the value as a primitive type, possibly performing an endianness swap to guarantee that the return value has the endianness of the native platform.
fn set(self: &mut Self, n: f32)Updates the value in place as a primitive type, possibly performing an endianness swap to guarantee that the stored value has the endianness
O.
impl<O> AsMut for F32<O>
fn as_mut(self: &mut Self) -> &mut [u8; 4]
impl<O> AsRef for F32<O>
fn as_ref(self: &Self) -> &[u8; 4]
impl<O> Default for F32<O>
fn default() -> F32<O>
impl<O> Freeze for F32<O>
impl<O> FromBytes for F32<O>
impl<O> FromZeros for F32<O>
impl<O> Immutable for F32<O>
impl<O> IntoBytes for F32<O>
impl<O> KnownLayout for F32<O>
impl<O> PartialEq for F32<O>
fn eq(self: &Self, other: &[u8; 4]) -> bool
impl<O> RefUnwindSafe for F32<O>
impl<O> Send for F32<O>
impl<O> StructuralPartialEq for F32<O>
impl<O> Sync for F32<O>
impl<O> TryFromBytes for F32<O>
impl<O> Unaligned for F32<O>
impl<O> Unpin for F32<O>
impl<O> UnsafeUnpin for F32<O>
impl<O> UnwindSafe for F32<O>
impl<O: $crate::clone::Clone> Clone for F32<O>
fn clone(self: &Self) -> F32<O>
impl<O: $crate::cmp::Eq> Eq for F32<O>
impl<O: $crate::cmp::PartialEq> PartialEq for F32<O>
fn eq(self: &Self, other: &F32<O>) -> bool
impl<O: $crate::hash::Hash> Hash for F32<O>
fn hash<__H: $crate::hash::Hasher>(self: &Self, state: &mut __H)
impl<O: $crate::marker::Copy> Copy for F32<O>
impl<O: ByteOrder> Add for F32<O>
fn add(self: Self, rhs: f32) -> F32<O>
impl<O: ByteOrder> Add for F32<O>
fn add(self: Self, rhs: F32<O>) -> F32<O>
impl<O: ByteOrder> AddAssign for F32<O>
fn add_assign(self: &mut Self, rhs: F32<O>)
impl<O: ByteOrder> AddAssign for F32<O>
fn add_assign(self: &mut Self, rhs: f32)
impl<O: ByteOrder> Debug for F32<O>
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl<O: ByteOrder> Display for F32<O>
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl<O: ByteOrder> Div for F32<O>
fn div(self: Self, rhs: f32) -> F32<O>
impl<O: ByteOrder> Div for F32<O>
fn div(self: Self, rhs: F32<O>) -> F32<O>
impl<O: ByteOrder> DivAssign for F32<O>
fn div_assign(self: &mut Self, rhs: F32<O>)
impl<O: ByteOrder> DivAssign for F32<O>
fn div_assign(self: &mut Self, rhs: f32)
impl<O: ByteOrder> From for F32<O>
fn from(x: f32) -> F32<O>
impl<O: ByteOrder> From for F32<O>
fn from(bytes: [u8; 4]) -> F32<O>
impl<O: ByteOrder> Mul for F32<O>
fn mul(self: Self, rhs: f32) -> F32<O>
impl<O: ByteOrder> Mul for F32<O>
fn mul(self: Self, rhs: F32<O>) -> F32<O>
impl<O: ByteOrder> MulAssign for F32<O>
fn mul_assign(self: &mut Self, rhs: F32<O>)
impl<O: ByteOrder> MulAssign for F32<O>
fn mul_assign(self: &mut Self, rhs: f32)
impl<O: ByteOrder> Neg for F32<O>
fn neg(self: Self) -> F32<O>
impl<O: ByteOrder> PartialEq for F32<O>
fn eq(self: &Self, other: &f32) -> bool
impl<O: ByteOrder> PartialOrd for F32<O>
fn partial_cmp(self: &Self, other: &Self) -> Option<Ordering>
impl<O: ByteOrder> Rem for F32<O>
fn rem(self: Self, rhs: f32) -> F32<O>
impl<O: ByteOrder> Rem for F32<O>
fn rem(self: Self, rhs: F32<O>) -> F32<O>
impl<O: ByteOrder> RemAssign for F32<O>
fn rem_assign(self: &mut Self, rhs: f32)
impl<O: ByteOrder> RemAssign for F32<O>
fn rem_assign(self: &mut Self, rhs: F32<O>)
impl<O: ByteOrder> Sub for F32<O>
fn sub(self: Self, rhs: f32) -> F32<O>
impl<O: ByteOrder> Sub for F32<O>
fn sub(self: Self, rhs: F32<O>) -> F32<O>
impl<O: ByteOrder> SubAssign for F32<O>
fn sub_assign(self: &mut Self, rhs: f32)
impl<O: ByteOrder> SubAssign for F32<O>
fn sub_assign(self: &mut Self, rhs: F32<O>)
impl<T> Any for F32<O>
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for F32<O>
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for F32<O>
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> CloneToUninit for F32<O>
unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)
impl<T> From for F32<O>
fn from(t: T) -> TReturns the argument unchanged.
impl<T, U> Into for F32<O>
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for F32<O>
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for F32<O>
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>