Struct I32

struct I32<O>(_, _)

A 32-bit signed integer stored in a given byte order.

I32 is like the native i32 type with two major differences: First, it has no alignment requirement (its alignment is 1). Second, the endianness of its memory layout is given by the type parameter O, which can be any type which implements ByteOrder. In particular, this refers to BigEndian, LittleEndian, NativeEndian, and NetworkEndian.

An I32 can be constructed using the new method, and its contained value can be obtained as a native i32 using the get method, or updated in place with the set method. In all cases, if the endianness O is not the same as the endianness of the current platform, an endianness swap will be performed in order to uphold the invariants that a) the layout of I32 has endianness O and that, b) the layout of i32 has the platform's native endianness.

I32 implements FromBytes, IntoBytes, and Unaligned, making it useful for parsing and serialization. See the module documentation for an example of how it can be used for parsing UDP packets.

Implementations

impl<O> I32<O>

const fn from_bytes(bytes: [u8; 4]) -> I32<O>

Constructs a new value from bytes which are already in O byte order.

const fn to_bytes(self: Self) -> [u8; 4]

Extracts the bytes of self without swapping the byte order.

The returned bytes will be in O byte order.

impl<O: ByteOrder> I32<O>

const fn new(n: i32) -> I32<O>

Constructs a new value, possibly performing an endianness swap to guarantee that the returned value has endianness O.

const fn get(self: Self) -> i32

Returns the value as a primitive type, possibly performing an endianness swap to guarantee that the return value has the endianness of the native platform.

fn set(self: &mut Self, n: i32)

Updates the value in place as a primitive type, possibly performing an endianness swap to guarantee that the stored value has the endianness O.

impl<O> AsMut for I32<O>

fn as_mut(self: &mut Self) -> &mut [u8; 4]

impl<O> AsRef for I32<O>

fn as_ref(self: &Self) -> &[u8; 4]

impl<O> Default for I32<O>

fn default() -> I32<O>

impl<O> Freeze for I32<O>

impl<O> FromBytes for I32<O>

impl<O> FromZeros for I32<O>

impl<O> Immutable for I32<O>

impl<O> IntoBytes for I32<O>

impl<O> KnownLayout for I32<O>

impl<O> Not for I32<O>

fn not(self: Self) -> I32<O>

impl<O> PartialEq for I32<O>

fn eq(self: &Self, other: &[u8; 4]) -> bool

impl<O> RefUnwindSafe for I32<O>

impl<O> Send for I32<O>

impl<O> StructuralPartialEq for I32<O>

impl<O> Sync for I32<O>

impl<O> TryFromBytes for I32<O>

impl<O> Unaligned for I32<O>

impl<O> Unpin for I32<O>

impl<O> UnsafeUnpin for I32<O>

impl<O> UnwindSafe for I32<O>

impl<O: $crate::clone::Clone> Clone for I32<O>

fn clone(self: &Self) -> I32<O>

impl<O: $crate::cmp::Eq> Eq for I32<O>

impl<O: $crate::cmp::PartialEq> PartialEq for I32<O>

fn eq(self: &Self, other: &I32<O>) -> bool

impl<O: $crate::hash::Hash> Hash for I32<O>

fn hash<__H: $crate::hash::Hasher>(self: &Self, state: &mut __H)

impl<O: $crate::marker::Copy> Copy for I32<O>

impl<O: ByteOrder> Add for I32<O>

fn add(self: Self, rhs: i32) -> I32<O>

impl<O: ByteOrder> Add for I32<O>

fn add(self: Self, rhs: I32<O>) -> I32<O>

impl<O: ByteOrder> AddAssign for I32<O>

fn add_assign(self: &mut Self, rhs: i32)

impl<O: ByteOrder> AddAssign for I32<O>

fn add_assign(self: &mut Self, rhs: I32<O>)

impl<O: ByteOrder> Binary for I32<O>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<O: ByteOrder> BitAnd for I32<O>

fn bitand(self: Self, rhs: i32) -> I32<O>

impl<O: ByteOrder> BitAnd for I32<O>

fn bitand(self: Self, rhs: I32<O>) -> I32<O>

impl<O: ByteOrder> BitAndAssign for I32<O>

fn bitand_assign(self: &mut Self, rhs: I32<O>)

impl<O: ByteOrder> BitAndAssign for I32<O>

fn bitand_assign(self: &mut Self, rhs: i32)

impl<O: ByteOrder> BitOr for I32<O>

fn bitor(self: Self, rhs: i32) -> I32<O>

impl<O: ByteOrder> BitOr for I32<O>

fn bitor(self: Self, rhs: I32<O>) -> I32<O>

impl<O: ByteOrder> BitOrAssign for I32<O>

fn bitor_assign(self: &mut Self, rhs: I32<O>)

impl<O: ByteOrder> BitOrAssign for I32<O>

fn bitor_assign(self: &mut Self, rhs: i32)

impl<O: ByteOrder> BitXor for I32<O>

fn bitxor(self: Self, rhs: i32) -> I32<O>

impl<O: ByteOrder> BitXor for I32<O>

fn bitxor(self: Self, rhs: I32<O>) -> I32<O>

impl<O: ByteOrder> BitXorAssign for I32<O>

fn bitxor_assign(self: &mut Self, rhs: I32<O>)

impl<O: ByteOrder> BitXorAssign for I32<O>

fn bitxor_assign(self: &mut Self, rhs: i32)

impl<O: ByteOrder> Debug for I32<O>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<O: ByteOrder> Display for I32<O>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<O: ByteOrder> Div for I32<O>

fn div(self: Self, rhs: I32<O>) -> I32<O>

impl<O: ByteOrder> Div for I32<O>

fn div(self: Self, rhs: i32) -> I32<O>

impl<O: ByteOrder> DivAssign for I32<O>

fn div_assign(self: &mut Self, rhs: i32)

impl<O: ByteOrder> DivAssign for I32<O>

fn div_assign(self: &mut Self, rhs: I32<O>)

impl<O: ByteOrder> From for I32<O>

fn from(x: i32) -> I32<O>

impl<O: ByteOrder> From for I32<O>

fn from(bytes: [u8; 4]) -> I32<O>

impl<O: ByteOrder> LowerHex for I32<O>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<O: ByteOrder> Mul for I32<O>

fn mul(self: Self, rhs: I32<O>) -> I32<O>

impl<O: ByteOrder> Mul for I32<O>

fn mul(self: Self, rhs: i32) -> I32<O>

impl<O: ByteOrder> MulAssign for I32<O>

fn mul_assign(self: &mut Self, rhs: i32)

impl<O: ByteOrder> MulAssign for I32<O>

fn mul_assign(self: &mut Self, rhs: I32<O>)

impl<O: ByteOrder> Neg for I32<O>

fn neg(self: Self) -> I32<O>

impl<O: ByteOrder> Octal for I32<O>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<O: ByteOrder> Ord for I32<O>

fn cmp(self: &Self, other: &Self) -> Ordering

impl<O: ByteOrder> PartialEq for I32<O>

fn eq(self: &Self, other: &i32) -> bool

impl<O: ByteOrder> PartialOrd for I32<O>

fn partial_cmp(self: &Self, other: &i32) -> Option<Ordering>

impl<O: ByteOrder> PartialOrd for I32<O>

fn partial_cmp(self: &Self, other: &Self) -> Option<Ordering>

impl<O: ByteOrder> Rem for I32<O>

fn rem(self: Self, rhs: I32<O>) -> I32<O>

impl<O: ByteOrder> Rem for I32<O>

fn rem(self: Self, rhs: i32) -> I32<O>

impl<O: ByteOrder> RemAssign for I32<O>

fn rem_assign(self: &mut Self, rhs: I32<O>)

impl<O: ByteOrder> RemAssign for I32<O>

fn rem_assign(self: &mut Self, rhs: i32)

impl<O: ByteOrder> Shl for I32<O>

fn shl(self: Self, rhs: i32) -> I32<O>

impl<O: ByteOrder> Shl for I32<O>

fn shl(self: Self, rhs: I32<O>) -> I32<O>

impl<O: ByteOrder> ShlAssign for I32<O>

fn shl_assign(self: &mut Self, rhs: I32<O>)

impl<O: ByteOrder> ShlAssign for I32<O>

fn shl_assign(self: &mut Self, rhs: i32)

impl<O: ByteOrder> Shr for I32<O>

fn shr(self: Self, rhs: i32) -> I32<O>

impl<O: ByteOrder> Shr for I32<O>

fn shr(self: Self, rhs: I32<O>) -> I32<O>

impl<O: ByteOrder> ShrAssign for I32<O>

fn shr_assign(self: &mut Self, rhs: I32<O>)

impl<O: ByteOrder> ShrAssign for I32<O>

fn shr_assign(self: &mut Self, rhs: i32)

impl<O: ByteOrder> Sub for I32<O>

fn sub(self: Self, rhs: I32<O>) -> I32<O>

impl<O: ByteOrder> Sub for I32<O>

fn sub(self: Self, rhs: i32) -> I32<O>

impl<O: ByteOrder> SubAssign for I32<O>

fn sub_assign(self: &mut Self, rhs: I32<O>)

impl<O: ByteOrder> SubAssign for I32<O>

fn sub_assign(self: &mut Self, rhs: i32)

impl<O: ByteOrder> TryFrom for I32<O>

fn try_from(x: i128) -> Result<I32<O>, TryFromIntError>

impl<O: ByteOrder> TryFrom for I32<O>

fn try_from(x: i64) -> Result<I32<O>, TryFromIntError>

impl<O: ByteOrder> UpperHex for I32<O>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<O: ByteOrder, P: ByteOrder> From for I32<P>

fn from(x: I16<O>) -> I32<P>

impl<O: ByteOrder, P: ByteOrder> TryFrom for I32<O>

fn try_from(x: I128<P>) -> Result<I32<O>, TryFromIntError>

impl<O: ByteOrder, P: ByteOrder> TryFrom for I32<O>

fn try_from(x: I64<P>) -> Result<I32<O>, TryFromIntError>

impl<T> Any for I32<O>

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for I32<O>

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for I32<O>

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for I32<O>

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for I32<O>

fn from(t: T) -> T

Returns the argument unchanged.

impl<T, U> Into for I32<O>

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for I32<O>

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for I32<O>

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>