pub struct Isize<O>(/* private fields */);Expand description
A word-sized signed integer stored in a given byte order.
Isize is like the native isize type with
two major differences: First, it has no alignment requirement (its alignment is 1).
Second, the endianness of its memory layout is given by the type parameter O,
which can be any type which implements ByteOrder. In particular, this refers
to BigEndian, LittleEndian, NativeEndian, and NetworkEndian.
An Isize can be constructed using
the new method, and its contained value can be obtained as a native
isize using the get method, or updated in place with
the set method. In all cases, if the endianness O is not the same as the
endianness of the current platform, an endianness swap will be performed in
order to uphold the invariants that a) the layout of Isize
has endianness O and that, b) the layout of isize has
the platform’s native endianness.
Isize implements FromBytes, IntoBytes, and Unaligned,
making it useful for parsing and serialization. See the module documentation for an
example of how it can be used for parsing UDP packets.
Implementations
Sourceimpl<O> Isize<O>
impl<O> Isize<O>
Sourcepub const ZERO: Isize<O>
pub const ZERO: Isize<O>
The value zero.
This constant should be preferred to constructing a new value
using new, as new may perform an endianness swap depending
on the endianness and platform.
Sourcepub const fn from_bytes(bytes: [u8; 4]) -> Isize<O>
pub const fn from_bytes(bytes: [u8; 4]) -> Isize<O>
Constructs a new value from bytes which are already in O byte
order.
Sourceimpl<O: ByteOrder> Isize<O>
impl<O: ByteOrder> Isize<O>
Sourcepub const fn new(n: isize) -> Isize<O>
Available on non-no_zerocopy_generic_bounds_in_const_fn_1_61_0 only.
pub const fn new(n: isize) -> Isize<O>
no_zerocopy_generic_bounds_in_const_fn_1_61_0 only.Constructs a new value, possibly performing an endianness
swap to guarantee that the returned value has endianness
O.