Struct zerocopy::byteorder::U16 [−][src]
A 16-bit unsigned integer
stored in O byte order.
U16 is like the native u16 type with
two major differences: First, it has no alignment requirement (its alignment is 1).
Second, the endianness of its memory layout is given by the type parameter O.
A U16 can be constructed using
the new method, and its contained value can be obtained as a native
u16 using the get method, or updated in place with
the set method. In all cases, if the endianness O is not the same as the
endianness of the current platform, an endianness swap will be performed in
order to uphold the invariants that a) the layout of U16
has endianness O and that, b) the layout of u16 has
the platform’s native endianness.
U16 implements FromBytes, AsBytes, and Unaligned,
making it useful for parsing and serialization. See the module documentation for an
example of how it can be used for parsing UDP packets.
Implementations
impl<O: ByteOrder> U16<O>[src]
pub const ZERO: U16<O>[src]
The value zero.
This constant should be preferred to constructing a new value
using new, as new may perform an endianness swap depending
on the endianness and platform.
pub const MAX_VALUE: U16<O>[src]
The maximum value.
This constant should be preferred to constructing a new value using
new, as new may perform an endianness swap depending on the
endianness O and the endianness of the platform.
pub fn new(n: u16) -> U16<O>[src]
Constructs a new value, possibly performing an endianness swap
to guarantee that the returned value has endianness O.
pub fn get(self) -> u16[src]
Returns the value as a primitive type, possibly performing an endianness swap to guarantee that the return value has the endianness of the native platform.
pub fn set(&mut self, n: u16)[src]
Updates the value in place as a primitive type, possibly
performing an endianness swap to guarantee that the stored value
has the endianness O.