Struct AllocError
struct AllocError
The error type of a failed allocation.
This type is intended to be deprecated in favor of the standard library's
AllocError type once it is stabilized. When that happens, this type will
be replaced by a type alias to the standard library type. We do not intend
to treat this as a breaking change; users who wish to avoid breakage should
avoid writing code which assumes that this is not such an alias. For
example, implementing the same trait for both types will result in an impl
conflict once this type is an alias.
Implementations
impl Clone for AllocError
fn clone(self: &Self) -> AllocError
impl Copy for AllocError
impl Debug for AllocError
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl Eq for AllocError
impl Freeze for AllocError
impl PartialEq for AllocError
fn eq(self: &Self, other: &AllocError) -> bool
impl RefUnwindSafe for AllocError
impl Send for AllocError
impl StructuralPartialEq for AllocError
impl Sync for AllocError
impl Unpin for AllocError
impl UnsafeUnpin for AllocError
impl UnwindSafe for AllocError
impl<T> Any for AllocError
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for AllocError
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for AllocError
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> CloneToUninit for AllocError
unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)
impl<T> From for AllocError
fn from(t: T) -> TReturns the argument unchanged.
impl<T, U> Into for AllocError
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for AllocError
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for AllocError
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>